//---------------------------------------------------------------------------
//  Windows hooks Sample Application - The DLL
//
//  This sample application demonstrates how to use Windows hooks.
//
//  This File contains the source code for the hooksdll
//
//  Author:     Kyle Marsh
//              Windows Developer Technology Group
//              Microsoft Corp.
//
//---------------------------------------------------------------------------


#include "windows.h"
#include "hooks.h"

//---------------------------------------------------------------------------
// Function declarations
//---------------------------------------------------------------------------

int   __export CALLBACK LibMain(HANDLE hModule, WORD wDataSeg, WORD cbHeapSize, LPSTR lpszCmdLine);
int   __export CALLBACK _WEP (int bSystemExit);
int   __export CALLBACK InstallFilter (int nHookIndex, int nCode );
int   __export CALLBACK KeyboardFunc (int nCode, WORD wParam, DWORD lParam );

//---------------------------------------------------------------------------
// Global Variables...
//---------------------------------------------------------------------------

HANDLE  hInstance;              // Global instance handle for  DLL
static HTASK hTaskClient;       //

//HWND    hwndMain;               // Main hwnd. We will get this from the App
//
//  My Hook States
//
int HookStates[NUMOFHOOKS] = { 0 } ; // State Table of my hooks

//
//  Hook Codes
//
int HookCodes[NUMOFHOOKS] = {
                               WH_KEYBOARD,
                            };

//
// Filter Function Addresses
//
FARPROC lpfnHookProcs[NUMOFHOOKS] = {
                                      (FARPROC) KeyboardFunc,
                                    };

//
// Hook Handles
//
HHOOK hhookHooks[NUMOFHOOKS];


//---------------------------------------------------------------------------
// LibMain
//---------------------------------------------------------------------------
int __export CALLBACK LibMain(HANDLE hModule, WORD wDataSeg, WORD cbHeapSize, LPSTR lpszCmdLine)
{
    hInstance = hModule;
    return 1;
}

#pragma alloc_text(FIXEDSEG, _WEP)

//---------------------------------------------------------------------------
// WEP
//---------------------------------------------------------------------------
int __export CALLBACK _WEP (int bSystemExit)
{
    return(1);
}

/*
//---------------------------------------------------------------------------
// InitHooksDll
//---------------------------------------------------------------------------
int __export CALLBACK InitHooksDll(HWND hwndMainWindow, int nWinLineHeight)
{
    hwndMain = hwndMainWindow;

    InitCalled = 1;
    return (0);
}

*/
//---------------------------------------------------------------------------
// InstallSysMsgFilter
//
// Install / Remove Filter function for the WH_SYSMSGFILTER
//
//---------------------------------------------------------------------------
int __export CALLBACK InstallFilter (int nHookIndex, int nCode )
{

     hTaskClient = GetCurrentTask();

     if ( nCode ) {
        hhookHooks[nHookIndex] = 
        SetWindowsHookEx(HookCodes[nHookIndex],
                         (HOOKPROC) lpfnHookProcs[nHookIndex],
                         hInstance,
                         NULL); 
                         
        HookStates[nHookIndex] = TRUE;
        MessageBox(NULL, 
                   INSTALLED, 
                   APPTITLE, 
                   MB_OK | MB_ICONINFORMATION);
     }
     else {
        UnhookWindowsHookEx(hhookHooks[nHookIndex]);
        HookStates[nHookIndex] = FALSE;
        MessageBox(NULL, 
                   UNINSTALLED, 
                   APPTITLE, 
                   MB_OK | MB_ICONINFORMATION);
     }
     return (TRUE);
}


//---------------------------------------------------------------------------
// KeyboardFunc
//
// Filter function for the WH_KEYBOARD
//
//---------------------------------------------------------------------------
int __export CALLBACK KeyboardFunc (int nCode, WORD wParam, DWORD lParam )
{
   MSG     msg;
   HWND    hActiveWnd;

   if ( nCode >= 0 ) {

      switch(wParam)
         {
          case VK_F12:

                if (GetAsyncKeyState(VK_F12) & 0x8000)
                   {
				                hActiveWnd = GetActiveWindow();
				                
				                if (NULL != hActiveWnd)
				                   {
				                    PostMessage(hActiveWnd,
				                                WM_CLOSE,
				                                0,
				                                0L);
				                   }
				                //
				                // Eat up following keystrokes (usually a WM_SYSKEYUP)
				                //
				
				                while
				                (
				                    PeekMessage
				                    (
				                        &msg, NULL, WM_KEYFIRST, WM_KEYLAST,
				                        PM_REMOVE | PM_NOYIELD
				                    )
				                )
				                {
				                    // Eat key msgs
				                }
				
				                return (TRUE);
				               }    
                break;           

          case VK_F10:
                
                if (GetAsyncKeyState(VK_F10) & 0x8000)
                   {
                
				
				                MessageBox(NULL,
				                           INFORMATION,
				                           APPTITLE,
				                           MB_OK | MB_ICONINFORMATION);
				                //
				                // Eat up following keystrokes (usually a WM_SYSKEYUP)
				                //
				
				                while
				                (
				                    PeekMessage
				                    (
				                        &msg, NULL, WM_KEYFIRST, WM_KEYLAST,
				                        PM_REMOVE | PM_NOYIELD
				                    )
				                )
				                {
				                    // Eat key msgs
				                }
				
				                return (TRUE);
				               } 
				            break;           

          case VK_F11 :
                
                // With a press of ctrl-shift-alt-x, pass a
                // message to the client app so it can
                // remove itself from memory. When the app
                // receives this message, HWND will be null
                // to indicate a task message as opposed to a
                // window message.
                //

                PostAppMessage(hTaskClient, WM_QUIT, 0, 0L);

                //
                // Eat up following keystrokes (usually a WM_SYSKEYUP)
                //

                while
                (
                    PeekMessage
                    (
                        &msg, NULL, WM_KEYFIRST, WM_KEYLAST,
                        PM_REMOVE | PM_NOYIELD
                    )
                )
                {
                    // Eat key msgs
                }

                return(1L);

                break;

          default :

                break;
        } //End switch (wParam)

   }  //End if (nCode >= 0)

   //
   // We looked at the message ... sort of processed it but since we are
   // looking we will pass all messages on to CallNextHookEx.
   //
   return( (int)CallNextHookEx(hhookHooks[0], nCode, wParam, lParam));
} //End KeyboardFunc ()

